<?php

namespace Drupal\migrate_cs\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\migrate\Plugin\MigrationPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Migrate Cron Scheduler settings for this site.
 */
class MigrateCsAdminSettingsForm extends ConfigFormBase implements ContainerInjectionInterface {

  /**
   * The migration plugin manager.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface
   */
  protected $migratePluginManager;

  /**
   * Constructs a new MigrateCsAdminSettingsForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\migrate\Plugin\MigrationPluginManagerInterface $plugin_manager
   *   The migration plugin manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MigrationPluginManagerInterface $plugin_manager) {
    parent::__construct($config_factory);
    $this->migratePluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.migration')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_cs_options';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['migrate_cs.options'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $options = $this->config('migrate_cs.options');
    $migrations = $this->migratePluginManager->getDefinitions();
    if ($migrations) {
      foreach ($migrations as $migration) {

        $mId = $migration['id'];

        $form[$mId] = [
          '#type' => 'details',
          '#title' => $migration['label'],
          '#description' => $this->t("Migrate cron scheduler settings for <em>@migration_id</em>", ['@migration_id' => $migrationId]),
          '#open' => TRUE,
        ];
        $form[$mId]["{$mId}_cs"] = [
          '#description' => $this->t('If checked, migration will run at cron.'),
          '#title' => $this->t('Run at cron'),
          '#type' => 'checkbox',
          '#default_value' => $options->get("{$mId}_cron"),
        ];

        $attributes = [
          'data-type' => 'number',
        ];
        if ($options->get("{$mId}_cs") == FALSE) {
          $attributes['disabled'] = TRUE;
        }

        $form[$mId]["{$mId}_interval"] = [
          '#maxlength' => 20,
          '#size' => 10,
          '#title' => 'Run at interval',
          '#description' => t("The interval (in seconds) the migration should run.<br/>If left empty or the value is lower then the cron interval - migration will run at each cron."),
          '#type' => 'number',
          '#default_value' => ($options->get("{$mId}_cs") == FALSE ? NULL : $options->get("{$mId}_interval")),
          '#attributes' => $attributes,
          '#states' => [
            'disabled' => [
              ':input[name="' . $mId . '_cs"]' => ['checked' => FALSE],
            ],
          ],
        ];

        $form[$mId]["{$mId}_type"] = [
          '#type' => 'Radio',
          '#title' => 't(Migration option)',
          '#default_value' => ($options->get("{$mId}_type") ? $options->get("{$mId}_type") : 'normal'),
          '#options' => [
            t('Normal') => 'normal', 
            t('Update') => 'update', 
            t('Sync') => 'sync',
          ],
        ];
      }
    }
    else {
      $form['migrations']['empty'] = [
        '#markup' => $this->t('There are no migrations.'),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $options = $this->config('migrate_cs.options');
    $migrations = $this->migratePluginManager->getDefinitions();
    if ($migrations) {
      foreach ($migrations as $migration) {
        $mId = $migration['id'];
        $options
          ->set("{$mId}_cs", $form_state->getValue("{$mId}_cs"))
          ->set("{$mId}_interval", $form_state->getValue("{$mId}_interval"))
          ->set("{$mId}_type", $form_state->getValue("{$mId}_type"))
          ->save();
      }
    }
  }

}
